<?php

class MeowPro_MGL_Core {
	private $item = 'Meow Gallery Pro';
	private $isEnqueued = false;
	private $core = null;

	private $collection_layout = 'bento';
	// TODO: Would be nice to detect if the collection is actually used on the current page.
	private $is_collection_used = true;

	public function __construct( $core ) {
		$this->core = $core;
		add_action( 'init', array( $this, 'license_enqueue' ) );
		if ( is_admin() ) {
			add_action( 'init', array( $this, 'enqueue_scripts' ) );
		}
		else {
			add_action( 'mgl_gallery_created', array( $this, 'enqueue_scripts' ), 10, 0 );
			add_action( 'mgl_collection_created', array( $this, 'enqueue_scripts' ), 10, 0 );
		}
		add_shortcode( 'meow-collection', array( $this, 'collection' ) );

		// Infinite
		if ( !MeowCommon_Helpers::is_rest() && Meow_MGL_Core::get_plugin_option( 'infinite', false ) ) {
			add_action( 'mgl_gallery_written', array( $this, 'rewrite_for_infinite' ), 10, 2 );
		}
	}

	function license_enqueue() {
		// Common behaviors, license, update system, etc.
		new MeowCommonPro_Licenser( MGL_PREFIX, MGL_ENTRY, MGL_DOMAIN, $this->item, MGL_VERSION );
	}

	function enqueue_scripts() {
		// Only need to load the scripts once.
		if ( $this->isEnqueued ) { 
			return;
		}
		$this->isEnqueued = true;


		$options = get_option( Meow_MGL_Core::get_plugin_option_name(), null );
		wp_localize_script('mgl-js', 'mgl_map',
			array(
				'default_engine' => ( $options['map_engine'] ?? "" ),
				'height' => ( $options['map_height'] ?? 400 ),
				'googlemaps' => array(
					'api_key' => ( $options['googlemaps_token'] ?? '' ),
					'style' => json_decode( ( $options['googlemaps_style'] ?? '' ) )
				),
				'mapbox' => array(
					'api_key' => ( $options['mapbox_token'] ?? '' ),
					'style' => json_decode( ( $options['mapbox_style'] ?? '' ) )
				),
				'maptiler' => array(
					'api_key' => ( $options['maptiler_token'] ?? '' ),
					'style' => json_decode( ( $options['maptiler_style'] ?? '' ) )
				)
			)
		);

		// Pro Style
		$physical_file = MGL_PATH . '/app/style-pro.min.css';
		$cache_buster = file_exists( $physical_file ) ? filemtime( $physical_file ) : MGL_VERSION;
		wp_enqueue_style( 'mgl-pro-css', plugin_dir_url( __DIR__ ) . 'app/style-pro.min.css', null, $cache_buster );
	}

	function rewrite_for_infinite( $html, $layout ) {
		if ( $layout == 'masonry' || $layout == 'carousel' )
			return $html;
		$xml = simplexml_load_string( $html );
		if ( empty( $xml ) )
			return $html;
		$subxml = $xml->xpath( '//img' );
		foreach ( $subxml as $s ) {
			$s['mgl-src'] = $s['src'];
			$s['mgl-srcset'] = $s['srcset'];
			$s['srcset'] = '';
			
			// New Lazy
			$s['loading'] = 'lazy';

			// Old Lazy
			//$s['class'] .= ' mgl-lazy';
			//$s['src'] = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7';
		}
		return $xml->asXml();
	}

	/**
	 * Foreach gallery id, get the the first image of the gallery, that will be used as thumbnail.
	 */
	function get_collection_thumbnails( $ids, $atts, $layout ) {
		$galleries = get_option( 'mgl_shortcodes', array() );
		$ids = explode( ',', $ids );
		$thumbnails = array();

		foreach ( $ids as $key => $id ) {
			if ( !isset( $galleries[$id] ) ) {
				$thumbnails[$key]['error'] = "missing";
				$thumbnails[$key]['id'] = $id;
				error_log( "⚠️ Meow Gallery: in collection $atts[id] the gallery $id was not found in the Gallery Manager." );
				continue;
			}

			$shortcode = $galleries[$id];
			$thumbnail_id = $shortcode['medias']['thumbnail_ids'][0];
			$thumbnails[$key]['gallery_id'] = $id;
			$thumbnails[$key]['media_id'] = $thumbnail_id;

			$thumbnails[$key]['img_src'] = wp_get_attachment_image_src( $thumbnail_id, 'large' )[0];
			$thumbnails[$key]['img_srcset'] = wp_get_attachment_image_srcset( $thumbnail_id, 'large' );
			$thumbnails[$key]['name'] = $shortcode['name'];
			$thumbnails[$key]['description'] = isset( $shortcode['description'] ) ? $shortcode['description'] : '';
		}

		return $thumbnails;
	}

	function get_wplr_collection_thumbnails( $wplr, $atts, $layout ) {
		$thumbnails = array();
		foreach ( $wplr as $key => $data ) {
			$thumbnails[$key]['wplr_collection_id'] = $data['id'];
			$thumbnails[$key]['media_id'] = $data['thumbnail'];

			$thumbnails[$key]['img_src'] = wp_get_attachment_image_src( $data['thumbnail'], 'large' )[0];
			$thumbnails[$key]['img_srcset'] = wp_get_attachment_image_srcset( $data['thumbnail'], 'large' );
			$thumbnails[$key]['name'] = $data['collection']->name;
			$thumbnails[$key]['description'] = '';
		}

		return $thumbnails;
	}

	function collection( $atts, $isPreview = false ) {
		$atts = apply_filters( 'shortcode_atts_collection', $atts, null, $atts );


		$gallery_ids = array();
		$layout = '';

		// Get the IDs
		if ( isset( $atts['id'] ) && isset( $atts['ids'] ) ) {
			error_log( "⚠️ Meow Gallery: in collection $atts[id] both 'id' and 'ids' attributes are used in the same shortcode. 'id' will be ignored." );
		}

		if ( isset( $atts['id'] ) && !isset( $atts['ids'] ) ) {
			$collections = get_option( 'mgl_collections', array() );
			if ( !isset( $collections[$atts['id']] ) ) {
				return "<p class='meow-error'><b>Meow Gallery:</b> This ID wasn't found in the Gallery Manager.</p>";
			}
			$atts['ids'] = $collections[$atts['id']]['galleries_ids'];
			$atts['layout'] = $collections[$atts['id']]['layout'];
		}

		if ( isset( $atts['ids'] ) ) {
			$gallery_ids = $atts['ids'];
			$gallery_ids = str_replace( ' ', '', $gallery_ids );
		}

		if ( !isset( $atts['id'] ) ) {
			$atts['id'] = 'temp__collection__' . uniqid();
		}

		// Filter the IDs
		$ids = is_array( $gallery_ids ) ? $gallery_ids : explode( ',', $gallery_ids );
		$ids = apply_filters( 'mgl_collection_ids', $ids, $atts );
		$gallery_ids = implode( ',', $ids );

		if ( empty( $gallery_ids ) && empty( $atts['wplr-thumbnails'] ) ) {
			return "<p class='meow-error'><b>Meow Gallery:</b> The collection is empty.</p>";
		}

		if ( $isPreview ) {
			$check = explode( ',', $gallery_ids );
			$check = array_slice( $check, 0, 5 );
			$gallery_ids = implode( ',', $check );
		}

		if ( isset( $atts['layout'] ) ){
			$layout = $atts['layout'];
		}
			
		if ( $layout === 'none' || $layout === '' ){
			error_log( "⚠️ Meow Gallery: in collection $atts[id] the layout is not defined. Default layout will be used.");
			$layout = 'bento'; //TODO: $this->get_option( 'collection_layout', 'bento' );
			$atts['layout'] = $layout;
		}

		$layoutClass = 'Meow_MGL_Builders_Collection_' . ucfirst( $layout );
		if ( !class_exists( $layoutClass ) ) {
			error_log( "Meow Gallery: Class $layoutClass does not exist." );
			return "<p class='meow-error'><b>Meow Gallery:</b> The collection layout $layout is not available in this version.</p>";
		}

		// if ( isset( $atts['titles'] ) && ( $atts['titles'] === false || $atts['titles'] === 'false' ) ) {
		// 	$atts['titles'] = 'none';	
		// }
		// if ( isset( $atts['captions'] ) && ( $atts['captions'] === false || $atts['captions'] === 'false' ) ) {
		// 	$atts['captions'] = 'none';	
		// }

		$this->collection_layout = $layout;
		do_action( 'mgl_collection_created', $atts, explode( ',', $gallery_ids ), $layout );

		if ( !empty( $atts['wplr-thumbnails'] ) ) {
			$collection_thumbnails = $this->get_wplr_collection_thumbnails( $atts['wplr-thumbnails'], $atts, $layout );
		}
		else {
			$collection_thumbnails = $this->get_collection_thumbnails( $gallery_ids, $atts, $layout );
		}

		// Get the class and data attributes
		$class = $this->core->get_mgl_root_class( $atts, ['mgl-collection-root'] );
		$data_atts = $this->core->get_data_as_json( $atts );
		$data_collection_thumbnails = $this->core->get_data_as_json( $collection_thumbnails );
		$html = '<div class="'. $class . '" data-collection-options="' . null . 
		'" data-collection-thumbnails="'. $data_collection_thumbnails .'" data-atts="'. $data_atts . '">';
		$html .= '<div class="mgl-collection-container"></div>';
		$html .= '</div>';
		return $html;
	}

}
